@@ -107,7 +107,6 @@ meta_compositor_process_event (MetaCompositor *compositor,
 #endif
 }
 
-#if GTK_CHECK_VERSION (3, 0, 0)
 cairo_surface_t *
 meta_compositor_get_window_surface (MetaCompositor *compositor,
                                     MetaWindow     *window)
@@ -121,21 +120,6 @@ meta_compositor_get_window_surface (MetaCompositor *compositor,
   return NULL;
 #endif
 }
-#else
-Pixmap
-meta_compositor_get_window_pixmap (MetaCompositor *compositor,
-                                   MetaWindow     *window)
-{
-#ifdef HAVE_COMPOSITE_EXTENSIONS
-  if (compositor && compositor->get_window_pixmap)
-    return compositor->get_window_pixmap (compositor, window);
-  else
-    return None;
-#else
-  return None;
-#endif
-}
-#endif  /* GTK_CHECK_VERSION */
 
 void
 meta_compositor_set_active_window (MetaCompositor *compositor,
