@@ -35,12 +35,6 @@
 #define _(x) dgettext (GETTEXT_PACKAGE, x)
 #define N_(x) x
 
-/* Silence Gtk{V,H}Box deprecations warnings */
-#if GTK_CHECK_VERSION (3, 0, 0)
-#define gtk_vbox_new(X, Y) gtk_box_new(GTK_ORIENTATION_VERTICAL, Y)
-#define gtk_hbox_new(X, Y) gtk_box_new (GTK_ORIENTATION_HORIZONTAL, Y)
-#endif
-
 /* We need to compute all different button arrangements
  * in terms of button location. We don't care about
  * different arrangements in terms of button function.
@@ -69,118 +63,136 @@ static MetaTheme *global_theme = NULL;
 static GtkWidget *previews[META_FRAME_TYPE_LAST*FONT_SIZE_LAST + BUTTON_LAYOUT_COMBINATIONS] = { NULL, };
 static double milliseconds_to_draw_frame = 0.0;
 
-static void run_position_expression_tests (void);
-#if 0
-static void run_position_expression_timings (void);
-#endif
 static void run_theme_benchmark (void);
 
+static const gchar *xml =
+  "<interface>"
+    "<menu id='menubar'>"
+      "<submenu>"
+        "<attribute name='label'>Windows</attribute>"
+        "<section>"
+          "<item>"
+            "<attribute name='label'>Dialog</attribute>"
+            "<attribute name='action'>theme-viewer.dialog1</attribute>"
+            "<attribute name='accel'>&lt;control&gt;d</attribute>"
+          "</item>"
+          "<item>"
+            "<attribute name='label'>Modal dialog</attribute>"
+            "<attribute name='action'>theme-viewer.dialog2</attribute>"
+          "</item>"
+          "<item>"
+            "<attribute name='label'>Utility</attribute>"
+            "<attribute name='action'>theme-viewer.utility</attribute>"
+            "<attribute name='accel'>&lt;control&gt;u</attribute>"
+          "</item>"
+          "<item>"
+            "<attribute name='label'>Splashscreen</attribute>"
+            "<attribute name='action'>theme-viewer.splashscreen</attribute>"
+            "<attribute name='accel'>&lt;control&gt;s</attribute>"
+          "</item>"
+          "<item>"
+            "<attribute name='label'>Top dock</attribute>"
+            "<attribute name='action'>theme-viewer.top-dock</attribute>"
+          "</item>"
+          "<item>"
+            "<attribute name='label'>Bottom dock</attribute>"
+            "<attribute name='action'>theme-viewer.bottom-dock</attribute>"
+          "</item>"
+          "<item>"
+            "<attribute name='label'>Left dock</attribute>"
+            "<attribute name='action'>theme-viewer.left-dock</attribute>"
+          "</item>"
+          "<item>"
+            "<attribute name='label'>Right dock</attribute>"
+            "<attribute name='action'>theme-viewer.right-dock</attribute>"
+          "</item>"
+          "<item>"
+            "<attribute name='label'>All docks</attribute>"
+            "<attribute name='action'>theme-viewer.all-docks</attribute>"
+          "</item>"
+          "<item>"
+            "<attribute name='label'>Desktop</attribute>"
+            "<attribute name='action'>theme-viewer.desktop</attribute>"
+          "</item>"
+        "</section>"
+      "</submenu>"
+    "</menu>"
+  "</interface>";
 
-static const gchar *menu_item_string =
-  "<ui>\n"
-    "<menubar>\n"
-      "<menu name='Windows' action='Windows'>\n"
-        "<menuitem name='Dialog' action='Dialog'/>\n"
-        "<menuitem name='Modal dialog' action='Modal dialog'/>\n"
-        "<menuitem name='Utility' action='Utility'/>\n"
-        "<menuitem name='Splashscreen' action='Splashscreen'/>\n"
-        "<menuitem name='Top dock' action='Top dock'/>\n"
-        "<menuitem name='Bottom dock' action='Bottom dock'/>\n"
-        "<menuitem name='Left dock' action='Left dock'/>\n"
-        "<menuitem name='Right dock' action='Right dock'/>\n"
-        "<menuitem name='Desktop' action='Desktop'/>\n"
-      "</menu>\n"
-    "</menubar>\n"
-    "<toolbar>\n"
-      "<separator/>\n"
-      "<toolitem name='New' action='New'/>\n"
-      "<toolitem name='Open' action='Open'/>\n"
-      "<toolitem name='Quit' action='Quit'/>\n"
-      "<separator/>\n"
-    "</toolbar>\n"
-  "</ui>\n";
-
-static GtkActionEntry menu_items[] =
+static GActionEntry theme_viewer_entries[] =
 {
-  { "Windows",           NULL, N_("_Windows"),       NULL,         NULL, NULL },
-  { "Dialog",            NULL, N_("_Dialog"),        "<control>d", NULL, NULL },
-  { "Modal dialog",      NULL, N_("_Modal dialog"),  NULL,         NULL, NULL },
-  { "Utility",           NULL, N_("_Utility"),       "<control>u", NULL, NULL },
-  { "Splashscreen",      NULL, N_("_Splashscreen"),  "<control>s", NULL, NULL },
-  { "Top dock",          NULL, N_("_Top dock"),      NULL,         NULL, NULL },
-  { "Bottom dock",       NULL, N_("_Bottom dock"),   NULL,         NULL, NULL },
-  { "Left dock",         NULL, N_("_Left dock"),     NULL,         NULL, NULL },
-  { "Right dock",        NULL, N_("_Right dock"),    NULL,         NULL, NULL },
-  { "All docks",         NULL, N_("_All docks"),     NULL,         NULL, NULL },
-  { "Desktop",           NULL, N_("Des_ktop"),       NULL,         NULL, NULL }
+  /* menubar */
+  { "dialog1",      NULL, NULL, NULL, NULL, {} },
+  { "dialog2",      NULL, NULL, NULL, NULL, {} },
+  { "utility",      NULL, NULL, NULL, NULL, {} },
+  { "splashscreen", NULL, NULL, NULL, NULL, {} },
+  { "top-dock",     NULL, NULL, NULL, NULL, {} },
+  { "bottom-dock",  NULL, NULL, NULL, NULL, {} },
+  { "left-dock",    NULL, NULL, NULL, NULL, {} },
+  { "right-dock",   NULL, NULL, NULL, NULL, {} },
+  { "all-docks",    NULL, NULL, NULL, NULL, {} },
+  { "desktop",      NULL, NULL, NULL, NULL, {} },
+  /* toolbar */
+  { "new",          NULL, NULL, NULL, NULL, {} },
+  { "open",         NULL, NULL, NULL, NULL, {} },
+  { "quit",         NULL, NULL, NULL, NULL, {} }
 };
 
-static GtkActionEntry tool_items[] =
+static GtkWidget *
+create_toolbar (void)
 {
-  { "New",      GTK_STOCK_NEW,  NULL,   NULL,
-    N_("Open another one of these windows"),            NULL },
-  { "Open",     GTK_STOCK_OPEN, NULL,   NULL,
-    N_("This is a demo button with an 'open' icon"),    NULL },
-  { "Quit",     GTK_STOCK_QUIT, NULL,   NULL,
-    N_("This is a demo button with a 'quit' icon"),     NULL }
-};
+  GtkWidget *toolbar;
+  GtkToolItem *item;
+
+  toolbar = gtk_toolbar_new ();
+
+  item = gtk_tool_button_new (gtk_image_new_from_icon_name ("document-new", GTK_ICON_SIZE_SMALL_TOOLBAR), NULL);
+  gtk_tool_item_set_tooltip_markup (item, "Open another one of these windows");
+  gtk_actionable_set_action_name (GTK_ACTIONABLE (item), "theme-viewer.new");
+  gtk_toolbar_insert (GTK_TOOLBAR (toolbar), item, -1);
+
+  item = gtk_tool_button_new (gtk_image_new_from_icon_name ("document-open", GTK_ICON_SIZE_SMALL_TOOLBAR), NULL);
+  gtk_tool_item_set_tooltip_markup (item, "This is a demo button with an 'open' icon");
+  gtk_actionable_set_action_name (GTK_ACTIONABLE (item), "theme-viewer.open");
+  gtk_toolbar_insert (GTK_TOOLBAR (toolbar), item, -1);
+
+  item = gtk_tool_button_new (gtk_image_new_from_icon_name ("application-exit", GTK_ICON_SIZE_SMALL_TOOLBAR), NULL);
+  gtk_tool_item_set_tooltip_markup (item, "This is a demo button with a 'quit' icon");
+  gtk_actionable_set_action_name (GTK_ACTIONABLE (item), "theme-viewer.quit");
+  gtk_toolbar_insert (GTK_TOOLBAR (toolbar), item, -1);
+
+  return toolbar;
+}
 
 static GtkWidget *
 normal_contents (void)
 {
-  GtkWidget *table;
-  GtkWidget *handlebox;
+  GtkWidget *grid;
   GtkWidget *statusbar;
   GtkWidget *contents;
   GtkWidget *sw;
-  GtkActionGroup *action_group;
-  GtkUIManager *ui_manager;
+  GtkBuilder *builder;
 
-  table = gtk_table_new (1, 4, FALSE);
-
-  /* Create the menubar
-   */
-
-  action_group = gtk_action_group_new ("mainmenu");
-  gtk_action_group_add_actions (action_group,
-                                menu_items,
-                                G_N_ELEMENTS (menu_items),
-                                NULL);
-  gtk_action_group_add_actions (action_group,
-                                tool_items,
-                                G_N_ELEMENTS (tool_items),
-                                NULL);
-
-  ui_manager = gtk_ui_manager_new ();
-
-  gtk_ui_manager_insert_action_group (ui_manager, action_group, 0);
+  grid = gtk_grid_new ();
+  builder = gtk_builder_new_from_string (xml, -1);
 
   /* create menu items */
-  gtk_ui_manager_add_ui_from_string (ui_manager, menu_item_string, -1, NULL);
+  GMenuModel *model = G_MENU_MODEL (gtk_builder_get_object (builder, "menubar"));
+  GtkWidget *menubar = gtk_menu_bar_new_from_model (model);
+  gtk_grid_attach (GTK_GRID (grid), menubar, 0, 0, 1, 1);
+  gtk_widget_set_hexpand (menubar, TRUE);
 
-  gtk_table_attach (GTK_TABLE (table),
-                    gtk_ui_manager_get_widget (ui_manager, "/ui/menubar"),
-                    /* X direction */          /* Y direction */
-                    0, 1,                      0, 1,
-                    GTK_EXPAND | GTK_FILL,     0,
-                    0,                         0);
-
-  handlebox = gtk_handle_box_new ();
-
-  gtk_container_add (GTK_CONTAINER (handlebox),
-                     gtk_ui_manager_get_widget (ui_manager, "/ui/toolbar"));
-
-  gtk_table_attach (GTK_TABLE (table),
-                    handlebox,
-                    /* X direction */       /* Y direction */
-                    0, 1,                   1, 2,
-                    GTK_EXPAND | GTK_FILL,  0,
-                    0,                      0);
+   /* Create the toolbar */
+  GtkWidget *toolbar = create_toolbar ();
+  gtk_grid_attach (GTK_GRID (grid), toolbar, 0, 1, 1, 1);
+  gtk_widget_set_hexpand (toolbar, TRUE);
 
   /* Create document
    */
 
   sw = gtk_scrolled_window_new (NULL, NULL);
+  gtk_widget_set_hexpand (sw, TRUE);
+  gtk_widget_set_vexpand (sw, TRUE);
 
   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw),
                                   GTK_POLICY_AUTOMATIC,
@@ -189,12 +201,9 @@ normal_contents (void)
   gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (sw),
                                        GTK_SHADOW_IN);
 
-  gtk_table_attach (GTK_TABLE (table),
-                    sw,
-                    /* X direction */       /* Y direction */
-                    0, 1,                   2, 3,
-                    GTK_EXPAND | GTK_FILL,  GTK_EXPAND | GTK_FILL,
-                    0,                      0);
+  gtk_grid_attach (GTK_GRID (grid),
+                   sw,
+                   0, 2, 1, 1);
 
   contents = gtk_text_view_new ();
   gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (contents),
@@ -206,18 +215,16 @@ normal_contents (void)
   /* Create statusbar */
 
   statusbar = gtk_statusbar_new ();
-  gtk_table_attach (GTK_TABLE (table),
-                    statusbar,
-                    /* X direction */       /* Y direction */
-                    0, 1,                   3, 4,
-                    GTK_EXPAND | GTK_FILL,  0,
-                    0,                      0);
+  gtk_widget_set_hexpand (statusbar, TRUE);
+  gtk_grid_attach (GTK_GRID (grid),
+                   statusbar,
+                   0, 3, 1, 1);
 
-  gtk_widget_show_all (table);
+  gtk_widget_show_all (grid);
 
-  g_object_unref (ui_manager);
+  g_object_unref (builder);
 
-  return table;
+  return grid;
 }
 
 static void
@@ -239,13 +246,9 @@ dialog_contents (void)
   GtkWidget *image;
   GtkWidget *button;
 
-  vbox = gtk_vbox_new (FALSE, 0);
+  vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
 
-#if GTK_CHECK_VERSION (3, 0, 0)
   action_area = gtk_button_box_new (GTK_ORIENTATION_HORIZONTAL);
-#else
-  action_area = gtk_hbutton_box_new ();
-#endif
 
   gtk_button_box_set_layout (GTK_BUTTON_BOX (action_area),
                              GTK_BUTTONBOX_END);
@@ -263,17 +266,13 @@ dialog_contents (void)
   label = gtk_label_new (_("This is a sample message in a sample dialog"));
   image = gtk_image_new_from_icon_name ("dialog-information",
                                         GTK_ICON_SIZE_DIALOG);
-#if GTK_CHECK_VERSION (3, 0, 0)
   gtk_widget_set_halign (image, GTK_ALIGN_CENTER);
   gtk_widget_set_valign (image, GTK_ALIGN_START);
-#else
-  gtk_misc_set_alignment (GTK_MISC (image), 0.5, 0.0);
-#endif
 
   gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
   gtk_label_set_selectable (GTK_LABEL (label), TRUE);
 
-  hbox = gtk_hbox_new (FALSE, 6);
+  hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
 
   gtk_box_pack_start (GTK_BOX (hbox), image,
                       FALSE, FALSE, 0);
@@ -293,11 +292,11 @@ dialog_contents (void)
 static GtkWidget*
 utility_contents (void)
 {
-  GtkWidget *table;
+  GtkWidget *grid;
   GtkWidget *button;
   int i, j;
 
-  table = gtk_table_new (3, 4, FALSE);
+  grid = gtk_grid_new ();
 
   i = 0;
   while (i < 3)
@@ -313,12 +312,12 @@ utility_contents (void)
 
           g_free (str);
 
-          gtk_table_attach (GTK_TABLE (table),
-                            button,
-                            /* X direction */       /* Y direction */
-                            i, i+1,                   j, j+1,
-                            GTK_EXPAND | GTK_FILL,  GTK_EXPAND | GTK_FILL,
-                            0,                      0);
+          gtk_widget_set_hexpand (button, TRUE);
+          gtk_widget_set_vexpand (button, TRUE);
+
+          gtk_grid_attach (GTK_GRID (grid),
+                           button,
+                           i, j, 1, 1);
 
           ++j;
         }
@@ -326,9 +325,9 @@ utility_contents (void)
       ++i;
     }
 
-  gtk_widget_show_all (table);
+  gtk_widget_show_all (grid);
 
-  return table;
+  return grid;
 }
 
 static GtkWidget*
@@ -343,19 +342,15 @@ menu_contents (void)
   gtk_frame_set_shadow_type (GTK_FRAME (frame),
                              GTK_SHADOW_OUT);
 
-  vbox = gtk_vbox_new (FALSE, 0);
+  vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
 
   i = 0;
   while (i < 10)
     {
       char *str = g_strdup_printf (_("Fake menu item %d\n"), i + 1);
       mi = gtk_label_new (str);
-#if GTK_CHECK_VERSION (3, 0, 0)
       gtk_widget_set_halign (mi, GTK_ALIGN_START);
       gtk_widget_set_valign (mi, GTK_ALIGN_CENTER);
-#else
-      gtk_misc_set_alignment (GTK_MISC (mi), 0.0, 0.5);
-#endif
       g_free (str);
       gtk_box_pack_start (GTK_BOX (vbox), mi, FALSE, FALSE, 0);
 
@@ -375,16 +370,17 @@ border_only_contents (void)
   GtkWidget *event_box;
   GtkWidget *vbox;
   GtkWidget *w;
-  GdkColor color;
+  GdkRGBA color;
 
   event_box = gtk_event_box_new ();
 
-  color.red = 40000;
+  color.red = 0.6;
   color.green = 0;
-  color.blue = 40000;
-  gtk_widget_modify_bg (event_box, GTK_STATE_NORMAL, &color);
+  color.blue = 0.6;
+  color.alpha = 1.0;
+  gtk_widget_override_background_color (event_box, 0, &color);
 
-  vbox = gtk_vbox_new (FALSE, 0);
+  vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
   gtk_container_set_border_width (GTK_CONTAINER (vbox), 3);
 
   w = gtk_label_new (_("Border-only window"));
@@ -484,13 +480,32 @@ get_window_flags (MetaFrameType type)
   return flags;
 }
 
+static void
+override_font (GtkWidget   *widget,
+               const gchar *font)
+{
+  gchar          *css;
+  GtkCssProvider *provider;
+
+  provider = gtk_css_provider_new ();
+
+  css = g_strdup_printf ("* { font: %s; }", font);
+  gtk_css_provider_load_from_data (provider, css, -1, NULL);
+  g_free (css);
+
+  gtk_style_context_add_provider (gtk_widget_get_style_context (widget),
+                                  GTK_STYLE_PROVIDER (provider),
+                                  GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
+  g_object_unref (provider);
+}
+
 static GtkWidget*
 preview_collection (int font_size,
-                    PangoFontDescription *base_desc)
+                    const PangoFontDescription *base_desc)
 {
   GtkWidget *box;
   GtkWidget *sw;
-  GdkColor desktop_color;
+  GdkRGBA desktop_color;
   int i;
   GtkWidget *eventbox;
 
@@ -499,31 +514,40 @@ preview_collection (int font_size,
                                   GTK_POLICY_AUTOMATIC,
                                   GTK_POLICY_AUTOMATIC);
 
-  box = gtk_vbox_new (FALSE, 0);
+  box = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
   gtk_box_set_spacing (GTK_BOX (box), 20);
   gtk_container_set_border_width (GTK_CONTAINER (box), 20);
 
   eventbox = gtk_event_box_new ();
   gtk_container_add (GTK_CONTAINER (eventbox), box);
 
-  gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (sw), eventbox);
+  gtk_container_add (GTK_CONTAINER (sw), eventbox);
 
-  desktop_color.red = 0x5144;
-  desktop_color.green = 0x75D6;
-  desktop_color.blue = 0xA699;
+  GSimpleActionGroup *action_group = g_simple_action_group_new ();
+  g_action_map_add_action_entries (G_ACTION_MAP (action_group),
+                                   theme_viewer_entries,
+                                   G_N_ELEMENTS (theme_viewer_entries),
+                                   NULL);
+  gtk_widget_insert_action_group (sw, "theme-viewer", G_ACTION_GROUP (action_group));
+  g_object_unref (action_group);
 
-  gtk_widget_modify_bg (eventbox, GTK_STATE_NORMAL, &desktop_color);
+  desktop_color.red = 0.32;
+  desktop_color.green = 0.46;
+  desktop_color.blue = 0.65;
+  desktop_color.alpha = 1.0;
+
+  gtk_widget_override_background_color (eventbox, 0, &desktop_color);
 
   i = 0;
   while (i < META_FRAME_TYPE_LAST)
     {
       const char *title = NULL;
       GtkWidget *contents;
-      GtkWidget *align;
-      double xalign, yalign;
+      GtkAlign xalign, yalign;
       GtkWidget *eventbox2;
       GtkWidget *preview;
       PangoFontDescription *font_desc;
+      gchar *font_string;
       double scale;
 
       eventbox2 = gtk_event_box_new ();
@@ -546,19 +570,19 @@ preview_collection (int font_size,
 
       if (i == META_FRAME_TYPE_MENU)
         {
-          xalign = 0.0;
-          yalign = 0.0;
+          xalign = GTK_ALIGN_START;
+          yalign = GTK_ALIGN_START;
         }
       else
         {
-          xalign = 0.5;
-          yalign = 0.5;
+          xalign = GTK_ALIGN_FILL;
+          yalign = GTK_ALIGN_FILL;
         }
 
-      align = gtk_alignment_new (0.0, 0.0, xalign, yalign);
-      gtk_container_add (GTK_CONTAINER (align), eventbox2);
+      gtk_widget_set_halign (eventbox2, xalign);
+      gtk_widget_set_valign (eventbox2, yalign);
 
-      gtk_box_pack_start (GTK_BOX (box), align, TRUE, TRUE, 0);
+      gtk_box_pack_start (GTK_BOX (box), eventbox2, TRUE, TRUE, 0);
 
       switch (font_size)
         {
@@ -580,7 +604,9 @@ preview_collection (int font_size,
           pango_font_description_set_size (font_desc,
                                            MAX (pango_font_description_get_size (base_desc) * scale, 1));
 
-          gtk_widget_modify_font (preview, font_desc);
+          font_string = pango_font_description_to_string (font_desc);
+          override_font (preview, font_string);
+          g_free (font_string);
 
           pango_font_description_free (font_desc);
         }
@@ -707,7 +733,7 @@ previews_of_button_layouts (void)
   static gboolean initted = FALSE;
   GtkWidget *box;
   GtkWidget *sw;
-  GdkColor desktop_color;
+  GdkRGBA desktop_color;
   int i;
   GtkWidget *eventbox;
 
@@ -722,26 +748,25 @@ previews_of_button_layouts (void)
                                   GTK_POLICY_AUTOMATIC,
                                   GTK_POLICY_AUTOMATIC);
 
-  box = gtk_vbox_new (FALSE, 0);
+  box = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
   gtk_box_set_spacing (GTK_BOX (box), 20);
   gtk_container_set_border_width (GTK_CONTAINER (box), 20);
 
   eventbox = gtk_event_box_new ();
   gtk_container_add (GTK_CONTAINER (eventbox), box);
 
-  gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (sw), eventbox);
+  gtk_container_add (GTK_CONTAINER (sw), eventbox);
 
-  desktop_color.red = 0x5144;
-  desktop_color.green = 0x75D6;
-  desktop_color.blue = 0xA699;
+  desktop_color.red = 0.32;
+  desktop_color.green = 0.46;
+  desktop_color.blue = 0.65;
+  desktop_color.alpha = 1.0;
 
-  gtk_widget_modify_bg (eventbox, GTK_STATE_NORMAL, &desktop_color);
+  gtk_widget_override_background_color (eventbox, 0, &desktop_color);
 
   i = 0;
   while (i < BUTTON_LAYOUT_COMBINATIONS)
     {
-      GtkWidget *align;
-      double xalign, yalign;
       GtkWidget *eventbox2;
       GtkWidget *preview;
       char *title;
@@ -761,13 +786,9 @@ previews_of_button_layouts (void)
       meta_preview_set_button_layout (META_PREVIEW (preview),
                                       &different_layouts[i]);
 
-      xalign = 0.5;
-      yalign = 0.5;
+      gtk_widget_set_halign (eventbox2, GTK_ALIGN_FILL);
 
-      align = gtk_alignment_new (0.0, 0.0, xalign, yalign);
-      gtk_container_add (GTK_CONTAINER (align), eventbox2);
-
-      gtk_box_pack_start (GTK_BOX (box), align, TRUE, TRUE, 0);
+      gtk_box_pack_start (GTK_BOX (box), eventbox2, TRUE, TRUE, 0);
 
       previews[META_FRAME_TYPE_LAST*FONT_SIZE_LAST + i] = preview;
 
@@ -796,7 +817,8 @@ main (int argc, char **argv)
 {
   GtkWidget *window;
   GtkWidget *collection;
-  GtkStyle *style;
+  GtkStyleContext *style;
+  PangoFontDescription *font_desc;
   GError *err;
   clock_t start, end;
   GtkWidget *notebook;
@@ -806,11 +828,6 @@ main (int argc, char **argv)
   textdomain(GETTEXT_PACKAGE);
   bind_textdomain_codeset(GETTEXT_PACKAGE, "UTF-8");
 
-  run_position_expression_tests ();
-#if 0
-  run_position_expression_timings ();
-#endif
-
   gtk_init (&argc, &argv);
 
   if (g_getenv ("MARCO_DEBUG") != NULL)
@@ -871,28 +888,29 @@ main (int argc, char **argv)
                     G_CALLBACK (gtk_main_quit), NULL);
 
   gtk_widget_realize (window);
-  style = gtk_widget_get_style (window);
+  style = gtk_widget_get_style_context (window);
+  gtk_style_context_get (style, GTK_STATE_FLAG_NORMAL, "font", &font_desc, NULL);
 
   g_assert (style);
-  g_assert (style->font_desc);
+  g_assert (font_desc);
 
   notebook = gtk_notebook_new ();
   gtk_container_add (GTK_CONTAINER (window), notebook);
 
   collection = preview_collection (FONT_SIZE_NORMAL,
-                                   style->font_desc);
+                                   font_desc);
   gtk_notebook_append_page (GTK_NOTEBOOK (notebook),
                             collection,
                             gtk_label_new (_("Normal Title Font")));
 
   collection = preview_collection (FONT_SIZE_SMALL,
-                                   style->font_desc);
+                                   font_desc);
   gtk_notebook_append_page (GTK_NOTEBOOK (notebook),
                             collection,
                             gtk_label_new (_("Small Title Font")));
 
   collection = preview_collection (FONT_SIZE_LARGE,
-                                   style->font_desc);
+                                   font_desc);
   gtk_notebook_append_page (GTK_NOTEBOOK (notebook),
                             collection,
                             gtk_label_new (_("Large Title Font")));
@@ -907,6 +925,8 @@ main (int argc, char **argv)
                             collection,
                             gtk_label_new (_("Benchmark")));
 
+  pango_font_description_free (font_desc);
+
   i = 0;
   while (i < (int) G_N_ELEMENTS (previews))
     {
@@ -943,8 +963,16 @@ get_flags (GtkWidget *widget)
 static int
 get_text_height (GtkWidget *widget)
 {
-  return meta_pango_font_desc_get_text_height (gtk_widget_get_style (widget)->font_desc,
-                                               gtk_widget_get_pango_context (widget));
+  GtkStyleContext      *style;
+  PangoFontDescription *font_desc;
+  int                   text_height;
+
+  style = gtk_widget_get_style_context (widget);
+  gtk_style_context_get (style, GTK_STATE_FLAG_NORMAL, "font", &font_desc, NULL);
+  text_height = meta_pango_font_desc_get_text_height (font_desc, gtk_widget_get_pango_context (widget));
+  pango_font_description_free (font_desc);
+
+  return text_height;
 }
 
 static PangoLayout*
@@ -961,7 +989,8 @@ static void
 run_theme_benchmark (void)
 {
   GtkWidget* widget;
-  GdkPixmap *pixmap;
+  cairo_surface_t *pixmap;
+  cairo_t *cr;
   int top_height, bottom_height, left_width, right_width;
   MetaButtonState button_states[META_BUTTON_TYPE_LAST] =
   {
@@ -1024,16 +1053,15 @@ run_theme_benchmark (void)
       /* Creating the pixmap in the loop is right, since
        * GDK does the same with its double buffering.
        */
-      pixmap = gdk_pixmap_new (gtk_widget_get_window (widget),
-                               client_width + left_width + right_width,
-                               client_height + top_height + bottom_height,
-                               -1);
+      pixmap = gdk_window_create_similar_surface (gtk_widget_get_window (widget),
+                                                  CAIRO_CONTENT_COLOR,
+                                                  client_width + left_width + right_width,
+                                                  client_height + top_height + bottom_height);
+      cr = cairo_create (pixmap);
 
       meta_theme_draw_frame (global_theme,
                              widget,
-                             pixmap,
-                             NULL,
-                             0, 0,
+                             cr,
                              META_FRAME_TYPE_NORMAL,
                              get_flags (widget),
                              client_width, client_height,
@@ -1044,7 +1072,8 @@ run_theme_benchmark (void)
                              meta_preview_get_mini_icon (),
                              meta_preview_get_icon ());
 
-      g_object_unref (G_OBJECT (pixmap));
+      cairo_destroy (cr);
+      cairo_surface_destroy (pixmap);
 
       ++i;
       client_width += inc;
@@ -1070,301 +1099,3 @@ run_theme_benchmark (void)
 #undef ITERATIONS
 }
 
-typedef struct
-{
-  GdkRectangle rect;
-  const char *expr;
-  int expected_x;
-  int expected_y;
-  MetaThemeError expected_error;
-} PositionExpressionTest;
-
-#define NO_ERROR -1
-
-static const PositionExpressionTest position_expression_tests[] = {
-  /* Just numbers */
-  { { 10, 20, 40, 50 },
-    "10", 20, 30, NO_ERROR },
-  { { 10, 20, 40, 50 },
-    "14.37", 24, 34, NO_ERROR },
-  /* Binary expressions with 2 ints */
-  { { 10, 20, 40, 50 },
-    "14 * 10", 150, 160, NO_ERROR },
-  { { 10, 20, 40, 50 },
-    "14 + 10", 34, 44, NO_ERROR },
-  { { 10, 20, 40, 50 },
-    "14 - 10", 14, 24, NO_ERROR },
-  { { 10, 20, 40, 50 },
-    "8 / 2", 14, 24, NO_ERROR },
-  { { 10, 20, 40, 50 },
-    "8 % 3", 12, 22, NO_ERROR },
-  /* Binary expressions with floats and mixed float/ints */
-  { { 10, 20, 40, 50 },
-    "7.0 / 3.5", 12, 22, NO_ERROR },
-  { { 10, 20, 40, 50 },
-    "12.1 / 3", 14, 24, NO_ERROR },
-  { { 10, 20, 40, 50 },
-    "12 / 2.95", 14, 24, NO_ERROR },
-  /* Binary expressions without whitespace after first number */
-  { { 10, 20, 40, 50 },
-    "14* 10", 150, 160, NO_ERROR },
-  { { 10, 20, 40, 50 },
-    "14+ 10", 34, 44, NO_ERROR },
-  { { 10, 20, 40, 50 },
-    "14- 10", 14, 24, NO_ERROR },
-  { { 10, 20, 40, 50 },
-    "8/ 2", 14, 24, NO_ERROR },
-  { { 10, 20, 40, 50 },
-    "7.0/ 3.5", 12, 22, NO_ERROR },
-  { { 10, 20, 40, 50 },
-    "12.1/ 3", 14, 24, NO_ERROR },
-  { { 10, 20, 40, 50 },
-    "12/ 2.95", 14, 24, NO_ERROR },
-  /* Binary expressions without whitespace before second number */
-  { { 10, 20, 40, 50 },
-    "14 *10", 150, 160, NO_ERROR },
-  { { 10, 20, 40, 50 },
-    "14 +10", 34, 44, NO_ERROR },
-  { { 10, 20, 40, 50 },
-    "14 -10", 14, 24, NO_ERROR },
-  { { 10, 20, 40, 50 },
-    "8 /2", 14, 24, NO_ERROR },
-  { { 10, 20, 40, 50 },
-    "7.0 /3.5", 12, 22, NO_ERROR },
-  { { 10, 20, 40, 50 },
-    "12.1 /3", 14, 24, NO_ERROR },
-  { { 10, 20, 40, 50 },
-    "12 /2.95", 14, 24, NO_ERROR },
-  /* Binary expressions without any whitespace */
-  { { 10, 20, 40, 50 },
-    "14*10", 150, 160, NO_ERROR },
-  { { 10, 20, 40, 50 },
-    "14+10", 34, 44, NO_ERROR },
-  { { 10, 20, 40, 50 },
-    "14-10", 14, 24, NO_ERROR },
-  { { 10, 20, 40, 50 },
-    "8/2", 14, 24, NO_ERROR },
-  { { 10, 20, 40, 50 },
-    "7.0/3.5", 12, 22, NO_ERROR },
-  { { 10, 20, 40, 50 },
-    "12.1/3", 14, 24, NO_ERROR },
-  { { 10, 20, 40, 50 },
-    "12/2.95", 14, 24, NO_ERROR },
-  /* Binary expressions with parentheses */
-  { { 10, 20, 40, 50 },
-    "(14) * (10)", 150, 160, NO_ERROR },
-  { { 10, 20, 40, 50 },
-    "(14) + (10)", 34, 44, NO_ERROR },
-  { { 10, 20, 40, 50 },
-    "(14) - (10)", 14, 24, NO_ERROR },
-  { { 10, 20, 40, 50 },
-    "(8) / (2)", 14, 24, NO_ERROR },
-  { { 10, 20, 40, 50 },
-    "(7.0) / (3.5)", 12, 22, NO_ERROR },
-  { { 10, 20, 40, 50 },
-    "(12.1) / (3)", 14, 24, NO_ERROR },
-  { { 10, 20, 40, 50 },
-    "(12) / (2.95)", 14, 24, NO_ERROR },
-  /* Lots of extra parentheses */
-  { { 10, 20, 40, 50 },
-    "(((14)) * ((10)))", 150, 160, NO_ERROR },
-  { { 10, 20, 40, 50 },
-    "((((14)))) + ((((((((10))))))))", 34, 44, NO_ERROR },
-  { { 10, 20, 40, 50 },
-    "((((((((((14 - 10))))))))))", 14, 24, NO_ERROR },
-  /* Binary expressions with variables */
-  { { 10, 20, 40, 50 },
-    "2 * width", 90, 100, NO_ERROR },
-  { { 10, 20, 40, 50 },
-    "2 * height", 110, 120, NO_ERROR },
-  { { 10, 20, 40, 50 },
-    "width - 10", 40, 50, NO_ERROR },
-  { { 10, 20, 40, 50 },
-    "height / 2", 35, 45, NO_ERROR },
-  /* More than two operands */
-  { { 10, 20, 40, 50 },
-    "8 / 2 + 5", 19, 29, NO_ERROR },
-  { { 10, 20, 40, 50 },
-    "8 * 2 + 5", 31, 41, NO_ERROR },
-  { { 10, 20, 40, 50 },
-    "8 + 2 * 5", 28, 38, NO_ERROR },
-  { { 10, 20, 40, 50 },
-    "8 + 8 / 2", 22, 32, NO_ERROR },
-  { { 10, 20, 40, 50 },
-    "14 / (2 + 5)", 12, 22, NO_ERROR },
-  { { 10, 20, 40, 50 },
-    "8 * (2 + 5)", 66, 76, NO_ERROR },
-  { { 10, 20, 40, 50 },
-    "(8 + 2) * 5", 60, 70, NO_ERROR },
-  { { 10, 20, 40, 50 },
-    "(8 + 8) / 2", 18, 28, NO_ERROR },
-  /* Errors */
-  { { 10, 20, 40, 50 },
-    "2 * foo", 0, 0, META_THEME_ERROR_UNKNOWN_VARIABLE },
-  { { 10, 20, 40, 50 },
-    "2 *", 0, 0, META_THEME_ERROR_FAILED },
-  { { 10, 20, 40, 50 },
-    "- width", 0, 0, META_THEME_ERROR_FAILED },
-  { { 10, 20, 40, 50 },
-    "5 % 1.0", 0, 0, META_THEME_ERROR_MOD_ON_FLOAT },
-  { { 10, 20, 40, 50 },
-    "1.0 % 5", 0, 0, META_THEME_ERROR_MOD_ON_FLOAT },
-  { { 10, 20, 40, 50 },
-    "! * 2", 0, 0, META_THEME_ERROR_BAD_CHARACTER },
-  { { 10, 20, 40, 50 },
-    "   ", 0, 0, META_THEME_ERROR_FAILED },
-  { { 10, 20, 40, 50 },
-    "() () (( ) ()) ((()))", 0, 0, META_THEME_ERROR_FAILED },
-  { { 10, 20, 40, 50 },
-    "(*) () ((/) ()) ((()))", 0, 0, META_THEME_ERROR_FAILED },
-  { { 10, 20, 40, 50 },
-    "2 * 5 /", 0, 0, META_THEME_ERROR_FAILED },
-  { { 10, 20, 40, 50 },
-    "+ 2 * 5", 0, 0, META_THEME_ERROR_FAILED },
-  { { 10, 20, 40, 50 },
-    "+ 2 * 5", 0, 0, META_THEME_ERROR_FAILED }
-};
-
-static void
-run_position_expression_tests (void)
-{
-#if 0
-  int i;
-  MetaPositionExprEnv env;
-
-  i = 0;
-  while (i < (int) G_N_ELEMENTS (position_expression_tests))
-    {
-      GError *err;
-      gboolean retval;
-      const PositionExpressionTest *test;
-      PosToken *tokens;
-      int n_tokens;
-      int x, y;
-
-      test = &position_expression_tests[i];
-
-      if (g_getenv ("META_PRINT_TESTS") != NULL)
-        g_print ("Test expression: \"%s\" expecting x = %d y = %d",
-                 test->expr, test->expected_x, test->expected_y);
-
-      err = NULL;
-
-      env.rect = meta_rect (test->rect.x, test->rect.y,
-                            test->rect.width, test->rect.height);
-      env.object_width = -1;
-      env.object_height = -1;
-      env.left_width = 0;
-      env.right_width = 0;
-      env.top_height = 0;
-      env.bottom_height = 0;
-      env.title_width = 5;
-      env.title_height = 5;
-      env.icon_width = 32;
-      env.icon_height = 32;
-      env.mini_icon_width = 16;
-      env.mini_icon_height = 16;
-      env.theme = NULL;
-
-      if (err == NULL)
-        {
-          retval = meta_parse_position_expression (tokens, n_tokens,
-                                                   &env,
-                                                   &x, &y,
-                                                   &err);
-        }
-
-      if (retval && err)
-        g_error (_("position expression test returned TRUE but set error"));
-      if (!retval && err == NULL)
-        g_error (_("position expression test returned FALSE but didn't set error"));
-      if (((int) test->expected_error) != NO_ERROR)
-        {
-          if (err == NULL)
-            g_error (_("Error was expected but none given"));
-          if (err->code != (int) test->expected_error)
-            g_error (_("Error %d was expected but %d given"),
-                     test->expected_error, err->code);
-        }
-      else
-        {
-          if (err)
-            g_error (_("Error not expected but one was returned: %s"),
-                     err->message);
-
-          if (x != test->expected_x)
-            g_error (_("x value was %d, %d was expected"), x, test->expected_x);
-
-          if (y != test->expected_y)
-            g_error (_("y value was %d, %d was expected"), y, test->expected_y);
-        }
-
-      if (err)
-        g_error_free (err);
-
-      meta_pos_tokens_free (tokens, n_tokens);
-      ++i;
-    }
-#endif
-}
-
-#if 0
-static void
-run_position_expression_timings (void)
-{
-  int i;
-  int iters;
-  clock_t start;
-  clock_t end;
-  MetaPositionExprEnv env;
-
-#define ITERATIONS 100000
-
-  start = clock ();
-
-  iters = 0;
-  i = 0;
-  while (iters < ITERATIONS)
-    {
-      const PositionExpressionTest *test;
-      int x, y;
-
-      test = &position_expression_tests[i];
-
-      env.x = test->rect.x;
-      env.y = test->rect.y;
-      env.width = test->rect.width;
-      env.height = test->rect.height;
-      env.object_width = -1;
-      env.object_height = -1;
-      env.left_width = 0;
-      env.right_width = 0;
-      env.top_height = 0;
-      env.bottom_height = 0;
-      env.title_width = 5;
-      env.title_height = 5;
-      env.icon_width = 32;
-      env.icon_height = 32;
-      env.mini_icon_width = 16;
-      env.mini_icon_height = 16;
-      env.theme = NULL;
-
-      meta_parse_position_expression (test->expr,
-                                      &env,
-                                      &x, &y, NULL);
-
-      ++iters;
-      ++i;
-      if (i == G_N_ELEMENTS (position_expression_tests))
-        i = 0;
-    }
-
-  end = clock ();
-
-  g_print (_("%d coordinate expressions parsed in %g seconds (%g seconds average)\n"),
-           ITERATIONS,
-           ((double)end - (double)start) / CLOCKS_PER_SEC,
-           ((double)end - (double)start) / CLOCKS_PER_SEC / (double) ITERATIONS);
-
-}
-#endif
