@@ -62,11 +62,6 @@ void meta_ui_init(int* argc, char*** argv)
 	{
 		meta_fatal ("Unable to open X display %s\n", XDisplayName (NULL));
 	}
-
-  /* We need to be able to fully trust that the window and monitor sizes
-   * that GDK reports corresponds to the X ones, so we disable the automatic
-   * scale handling */
-  gdk_x11_display_set_window_scale (gdk_display_get_default (), 1);
 }
 
 Display* meta_ui_get_display(void)
